package Heaps;
import SearchKeys.KeyedItem;

public class Heap {
  private int MAX_HEAP = 100;  //maximum size of heap;
  private KeyedItem[] items;    // array of heap items
  private int size;            // number of heap items

  public Heap() {  // default constructor
    items = new KeyedItem[MAX_HEAP];
    size = 0;
  }  // end default constructor
   
// heap operations:
  public boolean heapIsEmpty() {
  // Determines whether a heap is empty.
  // Precondition: None.
  // Postcondition: Returns true if the heap is empty;
  // otherwise returns false.
    return size == 0;
  } // end heapIsEmpty

  public void heapInsert(KeyedItem newItem) 
                         throws HeapException {
  // Inserts an item into a heap.
  // Precondition: newItem is the item to be inserted.
  // Postcondition: If the heap was not full, newItem is
  // in its proper position; otherwise HeapException is
  // thrown.

    if (size < MAX_HEAP) {
      // place the new item at the end of the heap
      items[size] = newItem;

      // trickle new item up to its proper position
      int place = size;
      int parent = (place - 1)/2;
      while ( (parent >= 0) && (items[place].getKey().compareTo
                               (items[parent].getKey()))<0){
        // swap items[place] and items[parent]
        KeyedItem temp = items[parent];
        items[parent] = items[place];
        items[place] = temp;
        
        place = parent;
        parent = (place - 1)/2;
      }  // end while

      ++size;
    }  // end if
    else {
      throw new HeapException("HeapException: Heap full");
    }  // end if
  } // end heapInsert

  public KeyedItem heapDelete() {
  // Retrieves and deletes the item in the root of a heap.
  // This item has the largest search key in the heap.
  // Precondition: None.
  // Postcondition: If the heap is not empty, returns the
  // item in the root of the heap and then deletes it. However,
  // if the heap is empty, removal is impossible and the
  // method returns null.
    KeyedItem rootItem = null;
    if (!heapIsEmpty()) {
      rootItem = items[0];
      items[0] = items[--size];
      heapRebuild(0);
    }  // end if
    return rootItem;
  } // end heapDelete

  protected void heapRebuild(int root) {
  // if the root is not a leaf and the root's search key 
  // is less than the larger of the search keys in the
  // root's children
    int child = 2 * root + 1;  // index of root's left 
                               // child, if any
    if ( child < size ) {
      // root is not a leaf, so it has a left child at child
      int rightChild = child + 1;  // index of right child, 
                                   // if any

      // if root has a right child, find larger child
      if ( (rightChild < size) && (items[rightChild].getKey().
                       compareTo(items[child].getKey()))<0) {
        child = rightChild;    // index of larger child
      } // end if

      // if the root's value is smaller than the
      // value in the larger child, swap values
      if (items[root].getKey().
                         compareTo(items[child].getKey())>0){
        KeyedItem temp = items[root];
        items[root] = items[child];
        items[child] = temp;
        // transform the new subtree into a heap
        heapRebuild(child);
      }  // end if
    }  // end if
    // if root is a leaf, do nothing
  } // end heapRebuild
  
} // end Heap
