package Tables;
import SearchKeys.KeyedItem;

// *********************************************************
// ADT table. 
// Sorted array-based implementation.
// Assumption: A table contains at most one item with a 
//             given search key at any time.
// *********************************************************

public class TableArrayBased implements TableInterface{
  final int MAX_TABLE = 100;        // maximum size of table;
  protected KeyedItem[] items;      // table items
  private int size;                 // table size

  public TableArrayBased() {
    items = new KeyedItem[MAX_TABLE];
    size = 0;
  }  // default constructor

  public boolean tableIsEmpty() {
    return size==0;
  } // end tableIsEmpty

  public int tableLength() {
    return size;
  }  // end tableLength

  public void tableInsert(KeyedItem newItem) 
                     throws TableException {
  // Calls: position.
    if (size < MAX_TABLE) {  
      // there is room to insert;
      // locate the position where newItem belongs
      int spot = position(newItem.getKey());
      if ((spot < size) &&
          (items[spot].getKey()).compareTo(newItem.getKey())==0) {
          // we have found a duplicate key
          throw new TableException("Table Exception: " + 
                    "Insertion failed, duplicate key item");
      }
      else {
        // shift up to make room for the new item
        for (int index = size-1; index >= spot; --index) {
          items[index+1] = items[index];
        } //end for
        // make the insertion
        items[spot] = newItem;
        ++size;
      }  // end if
    }
    else {
      throw new TableException("TableException: Table full");
    } // end if
  }  // end tableInsert

  public boolean tableDelete(Comparable searchKey) {
  // Calls: position.
    // locate the position where searchKey exists/belongs
    int spot = position(searchKey);
    // is searchKey present in the table?
    boolean success = (spot <= size) && 
            (items[spot].getKey().compareTo(searchKey)==0);
    if (success) {
      // searchKey in table
      --size;  // delete the item
      // shift down to fill the gap
      for (int index = spot; index < size; ++index) {
        items[index] = items[index+1];
      } //end for
    }  // end if
    return success;
  }  // end tableDelete

  public KeyedItem tableRetrieve(Comparable searchKey) {
  // Calls: position.

    // locate the position where searchKey exists/belongs
    int spot = position(searchKey);
    // is searchKey present in table?
    boolean success = (spot <= size) &&
            (items[spot].getKey().compareTo(searchKey)==0);
    if (success) {
      return items[spot];  // item present; retrieve it
    }
    else {
      return null;
    }  // end if
  }  // end tableRetrieve

  protected int position(Comparable searchKey) {
  // Finds the position of a table item or its insertion 
  // point.
  // Precondition: searchKey is the value of the search key
  // sought in the table.
  // Postcondition: Returns the index (between 0 and 
  // size - 1) of the item in the table whose search key 
  // equals searchKey. If no such item exists, returns the 
  // position (between 0 and size) that the item would 
  // occupy if inserted into the table. The table is 
  // unchanged.
    int pos = 0;
    while ((pos < size) && 
           (searchKey.compareTo(items[pos].getKey()) > 0)) {
      pos++;
    }  // end while
    return pos;
  }  // end position

}  // end TableArrayBased