package Tables;
import BinaryTrees.*;
import SearchKeys.KeyedItem;

// Assumes that the binary search tree created in Chapter 10
// is contained in a package called BinaryTrees.

// ********************************************************
// Implementation of a table using a binary search tree.
// Assumption: A table contains at most one item with a 
//             given search key at any time.
// ********************************************************

public class TableBSTBased implements TableInterface {
  protected BinarySearchTree bst; // binary search tree that
                             // contains the tables items
  protected int size;          // number of items in the table

  public TableBSTBased() {  // default constructor
    bst = new BinarySearchTree();
    size = 0;
  }  // end default constructor
  
// table operations:
  public boolean tableIsEmpty() {
    return size == 0;
  }  // end tableIsEmpty
   
  public int tableLength() {
    return size;
  }  // end tableLength
  
  public void tableInsert(KeyedItem newItem) 
                          throws TableException {
    if (bst.retrieve(newItem.getKey()) == null) { 
      bst.insert(newItem);
      ++size;
    }
    else {
      throw new TableException("Table Exception: Insertion" 
                              + " failed, duplicate key item");
    } // end if
  }  // end tableInsert

  public KeyedItem tableRetrieve(Comparable searchKey) {
    return bst.retrieve(searchKey);
  }  // end tableRetrieve

  public boolean tableDelete(Comparable searchKey) {
    try {
      bst.delete(searchKey);
    }  // end try
    catch (TreeException e) {
      return false;
    }  //end catch
    --size;
    return true;
  }  // end tableDelete

  protected void setSize(int newSize) {
    size = newSize;
  }  // end setSize

}  // end TableBSTBased