// This version of IncomeStatistics includes the code for readData
// that appears on pages 30-31
import java.io.*;
import java.util.*;

public class IncomeStatisticsV2 {
  final static int LOW_END = 10;    // low end of incomes
  final static int HIGH_END = 100;  // high end of incomes
  final static int TABLE_SIZE = HIGH_END - LOW_END + 1;

  int[] incomeData;    // used to store the income data,
      // incomeData[G] stores the total number of
      // people that fall into the G-thousand-dollar group

  public IncomeStatisticsV2() {
    incomeData = new int[TABLE_SIZE];
  }  // end constructor

  // readData pages 30-31
  public boolean readData() throws IOException{
  // ------------------------------------------------------
  // Reads and organizes income statistics.
  // Precondition: The calling program gives directions and
  // prompts the user. Each input line contains exactly two 
  // integers in the form G N, where N is the number of people
  // with an income in the G-thousand-dollar group and
  // LOW_END <= G <= HIGH_END. An input line with values of zero for 
  // both G and N terminates the input.
  // Postcondition: incomeData[G-LOW_END] = total number of people
  // with an income in the G-thousand-dollar group. The values 
  // read are displayed. If either G or N is erroneous (G and 
  // N are not both 0, and either G < LOW_END, G > HIGH_END, 
  // or N < 0), the method ignores the data line, sets the 
  // return value to false, and continues. In this case, 
  // the calling program should take action. 
  // The return value is true if the data is error free.
  // -------------------------------------------------------
    int group, number;           // input values
    boolean dataCorrect = true;  // no data error found as yet
    BufferedReader stdin = new BufferedReader(
                           new InputStreamReader(System.in));
    StringTokenizer inputLine;

    for (group = LOW_END; group <= HIGH_END; ++group) { // clear array
      incomeData[index(group)] = 0;
    }  // end for
    inputLine = new StringTokenizer(stdin.readLine());
    group = Integer.parseInt(inputLine.nextToken());
    number = Integer.parseInt(inputLine.nextToken());
    while ((group != 0) || (number != 0)) {
      // Invariant: group and number are not both 0
      System.out.println("Income group " + group + 
                         " contains " + number + " people.");
      if ((group >= LOW_END) && (group <= HIGH_END) && (number >= 0)) {
        // input data is valid -- add it to tally
        incomeData[index(group)] += number;
      } 
      else {
        // error in input data: set error flag and ignore line
        dataCorrect = false;
      }
      inputLine = new StringTokenizer(stdin.readLine());
      group = Integer.parseInt(inputLine.nextToken());
      number = Integer.parseInt(inputLine.nextToken());
    }  // end while
    return dataCorrect;
  }  // end readData

  private int index(int group) {
  // Returns the array index corresponding to group number.
    return group - LOW_END;
  }  // end index

// other methods for class IncomeStatistics would follow

} // end IncomeStatistics
